@@ -2,19 +2,23 @@
  * 
  * This file is part of JSON-GLib
  * Copyright (C) 2007  OpenedHand Ltd.
+ * Copyright (C) 2009  Intel Corp.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
  * Author:
- *   Emmanuele Bassi  <ebassi@openedhand.com>
+ *   Emmanuele Bassi  <ebassi@linux.intel.com>
  */
 
 /**
@@ -31,6 +35,8 @@
 
 #include <string.h>
 
+#include "json-types-private.h"
+
 #include "json-marshal.h"
 #include "json-parser.h"
 #include "json-scanner.h"
@@ -54,7 +60,10 @@ struct _JsonParserPrivate
   GError *last_error;
 
   guint has_assignment : 1;
+  guint is_filename    : 1;
+
   gchar *variable_name;
+  gchar *filename;
 };
 
 static const gchar symbol_names[] =
@@ -68,10 +77,10 @@ static const struct
   guint name_offset;
   guint token;
 } symbols[] = {
-  {  0, JSON_TOKEN_TRUE },
+  {  0, JSON_TOKEN_TRUE  },
   {  5, JSON_TOKEN_FALSE },
-  { 11, JSON_TOKEN_NULL },
-  { 16, JSON_TOKEN_VAR }
+  { 11, JSON_TOKEN_NULL  },
+  { 16, JSON_TOKEN_VAR   }
 };
 
 static const guint n_symbols = G_N_ELEMENTS (symbols);
@@ -108,6 +117,7 @@ json_parser_clear (JsonParser *parser)
   JsonParserPrivate *priv = parser->priv;
 
   g_free (priv->variable_name);
+  priv->variable_name = NULL;
 
   if (priv->last_error)
     {
@@ -130,6 +140,17 @@ json_parser_dispose (GObject *gobject)
   G_OBJECT_CLASS (json_parser_parent_class)->dispose (gobject);
 }
 
+static void
+json_parser_finalize (GObject *gobject)
+{
+  JsonParserPrivate *priv = JSON_PARSER (gobject)->priv;
+
+  g_free (priv->variable_name);
+  g_free (priv->filename);
+
+  G_OBJECT_CLASS (json_parser_parent_class)->finalize (gobject);
+}
+
 static void
 json_parser_class_init (JsonParserClass *klass)
 {
@@ -138,6 +159,7 @@ json_parser_class_init (JsonParserClass *klass)
   g_type_class_add_private (klass, sizeof (JsonParserPrivate));
 
   gobject_class->dispose = json_parser_dispose;
+  gobject_class->finalize = json_parser_finalize;
 
   /**
    * JsonParser::parse-start:
@@ -304,6 +326,9 @@ json_parser_init (JsonParser *parser)
 
   priv->has_assignment = FALSE;
   priv->variable_name = NULL;
+
+  priv->is_filename = FALSE;
+  priv->filename = FALSE;
 }
 
 static guint
@@ -337,6 +362,7 @@ json_parse_array (JsonParser *parser,
         {
           /* swallow the comma */
           token = json_scanner_get_next_token (scanner);
+
           continue;
         }
 
@@ -356,6 +382,7 @@ json_parse_array (JsonParser *parser,
             {
               json_node_free (node);
               json_array_unref (array);
+
               return token;
             }
 
@@ -371,7 +398,11 @@ json_parse_array (JsonParser *parser,
             break;
 
           if (token != G_TOKEN_COMMA)
-            return G_TOKEN_RIGHT_BRACE;
+            {
+              json_array_unref (array);
+
+              return G_TOKEN_RIGHT_BRACE;
+            }
 
           continue;
         }
@@ -392,6 +423,7 @@ json_parse_array (JsonParser *parser,
             {
               json_node_free (node);
               json_array_unref (array);
+
               return token;
             }
 
@@ -407,7 +439,11 @@ json_parse_array (JsonParser *parser,
             break;
 
           if (token != G_TOKEN_COMMA)
-            return G_TOKEN_RIGHT_BRACE;
+            {
+              json_array_unref (array);
+
+              return G_TOKEN_RIGHT_BRACE;
+            }
 
           continue;
         }
@@ -424,6 +460,8 @@ json_parse_array (JsonParser *parser,
             }
           else
             {
+              json_array_unref (array);
+
               return G_TOKEN_INT;
             }
         }
@@ -450,8 +488,7 @@ json_parse_array (JsonParser *parser,
         case JSON_TOKEN_TRUE:
         case JSON_TOKEN_FALSE:
           node = json_node_new (JSON_NODE_VALUE);
-          json_node_set_boolean (node, token == JSON_TOKEN_TRUE ? TRUE
-                                                                : FALSE);
+          json_node_set_boolean (node, token == JSON_TOKEN_TRUE ? TRUE : FALSE);
           break;
 
         case JSON_TOKEN_NULL:
@@ -459,6 +496,7 @@ json_parse_array (JsonParser *parser,
           break;
 
         default:
+          json_array_unref (array);
           return G_TOKEN_RIGHT_BRACE;
         }
 
@@ -474,7 +512,11 @@ json_parse_array (JsonParser *parser,
 
       token = json_scanner_get_next_token (scanner);
       if (token != G_TOKEN_COMMA && token != G_TOKEN_RIGHT_BRACE)
-        return G_TOKEN_RIGHT_BRACE;
+        {
+          json_array_unref (array);
+
+          return G_TOKEN_RIGHT_BRACE;
+        }
     }
 
   json_node_take_array (priv->current_node, array);
@@ -516,6 +558,7 @@ json_parse_object (JsonParser *parser,
         {
           /* swallow the comma */
           token = json_scanner_get_next_token (scanner);
+
           continue;
         }
 
@@ -528,6 +571,7 @@ json_parse_object (JsonParser *parser,
             {
               g_free (name);
               json_object_unref (object);
+
               return ':';
             }
           else
@@ -540,6 +584,7 @@ json_parse_object (JsonParser *parser,
       if (!name)
         {
           json_object_unref (object);
+
           return G_TOKEN_STRING;
         }
 
@@ -562,10 +607,11 @@ json_parse_object (JsonParser *parser,
                 json_node_free (node);
 
               json_object_unref (object);
+
               return token;
             }
 
-          json_object_add_member (object, name, node);
+          json_object_set_member (object, name, node);
           node->parent = priv->current_node;
 
           g_signal_emit (parser, parser_signals[OBJECT_MEMBER], 0,
@@ -579,7 +625,11 @@ json_parse_object (JsonParser *parser,
             break;
 
           if (token != G_TOKEN_COMMA)
-            return G_TOKEN_RIGHT_CURLY;
+            {
+              json_object_unref (object);
+
+              return G_TOKEN_RIGHT_CURLY;
+            }
 
           continue;
         }
@@ -600,10 +650,11 @@ json_parse_object (JsonParser *parser,
               g_free (name);
               json_node_free (node);
               json_object_unref (object);
+
               return token;
             }
 
-          json_object_add_member (object, name, node);
+          json_object_set_member (object, name, node);
           node->parent = priv->current_node;
           
           g_signal_emit (parser, parser_signals[OBJECT_MEMBER], 0,
@@ -617,7 +668,11 @@ json_parse_object (JsonParser *parser,
             break;
 
           if (token != G_TOKEN_COMMA)
-            return G_TOKEN_RIGHT_CURLY;
+            {
+              json_object_unref (object);
+
+              return G_TOKEN_RIGHT_CURLY;
+            }
 
           continue;
         }
@@ -633,6 +688,9 @@ json_parse_object (JsonParser *parser,
             }
           else
             {
+              g_free (name);
+              json_object_unref (object);
+
               return G_TOKEN_INT;
             }
         }
@@ -659,8 +717,7 @@ json_parse_object (JsonParser *parser,
         case JSON_TOKEN_TRUE:
         case JSON_TOKEN_FALSE:
           node = json_node_new (JSON_NODE_VALUE);
-          json_node_set_boolean (node, token == JSON_TOKEN_TRUE ? TRUE
-                                                                : FALSE);
+          json_node_set_boolean (node, token == JSON_TOKEN_TRUE ? TRUE : FALSE);
           break;
 
         case JSON_TOKEN_NULL:
@@ -668,12 +725,14 @@ json_parse_object (JsonParser *parser,
           break;
 
         default:
+          g_free (name);
+          json_object_unref (object);
           return G_TOKEN_SYMBOL;
         }
 
       if (node)
         {
-          json_object_add_member (object, name, node);
+          json_object_set_member (object, name, node);
           node->parent = priv->current_node;
           
           g_signal_emit (parser, parser_signals[OBJECT_MEMBER], 0,
@@ -696,8 +755,8 @@ json_parse_object (JsonParser *parser,
 }
 
 static guint
-json_parse_statement (JsonParser *parser,
-                      JsonScanner   *scanner)
+json_parse_statement (JsonParser  *parser,
+                      JsonScanner *scanner)
 {
   JsonParserPrivate *priv = parser->priv;
   guint token;
@@ -757,6 +816,7 @@ json_parse_statement (JsonParser *parser,
 
     case JSON_TOKEN_NULL:
       priv->root = priv->current_node = json_node_new (JSON_NODE_NULL);
+      json_scanner_get_next_token (scanner);
       return G_TOKEN_NONE;
 
     case JSON_TOKEN_TRUE:
@@ -764,6 +824,7 @@ json_parse_statement (JsonParser *parser,
       priv->root = priv->current_node = json_node_new (JSON_NODE_VALUE);
       json_node_set_boolean (priv->current_node,
                              token == JSON_TOKEN_TRUE ? TRUE : FALSE);
+      json_scanner_get_next_token (scanner);
       return G_TOKEN_NONE;
 
     case '-':
@@ -792,6 +853,7 @@ json_parse_statement (JsonParser *parser,
                 return G_TOKEN_INT;
               }
 
+            json_scanner_get_next_token (scanner);
             return G_TOKEN_NONE;
           }
         else
@@ -803,12 +865,15 @@ json_parse_statement (JsonParser *parser,
     case G_TOKEN_FLOAT:
     case G_TOKEN_STRING:
       priv->root = priv->current_node = json_node_new (JSON_NODE_VALUE);
+
       if (token == G_TOKEN_INT)
         json_node_set_int (priv->current_node, scanner->value.v_int);
       else if (token == G_TOKEN_FLOAT)
         json_node_set_double (priv->current_node, scanner->value.v_float);
       else
         json_node_set_string (priv->current_node, scanner->value.v_string);
+
+      json_scanner_get_next_token (scanner);
       return G_TOKEN_NONE;
 
     default:
@@ -823,6 +888,7 @@ json_scanner_msg_handler (JsonScanner *scanner,
                           gboolean  is_error)
 {
   JsonParser *parser = scanner->user_data;
+  JsonParserPrivate *priv = parser->priv;
 
   if (is_error)
     {
@@ -830,7 +896,8 @@ json_scanner_msg_handler (JsonScanner *scanner,
 
       g_set_error (&error, JSON_PARSER_ERROR,
                    JSON_PARSER_ERROR_PARSE,
-                   "Parse error on line %d: %s",
+                   "%s:%d: Parse error: %s",
+                   priv->is_filename ? priv->filename : "<none>",
                    scanner->line,
                    message);
       
@@ -838,7 +905,10 @@ json_scanner_msg_handler (JsonScanner *scanner,
       g_signal_emit (parser, parser_signals[ERROR], 0, error);
     }
   else
-    g_warning ("Line %d: %s", scanner->line, message);
+    g_warning ("%s:%d: Parse error: %s",
+               priv->is_filename ? priv->filename : "<none>",
+               scanner->line,
+               message);
 }
 
 static JsonScanner *
@@ -877,89 +947,24 @@ json_parser_new (void)
   return g_object_new (JSON_TYPE_PARSER, NULL);
 }
 
-/**
- * json_parser_load_from_file:
- * @parser: a #JsonParser
- * @filename: the path for the file to parse
- * @error: return location for a #GError, or %NULL
- *
- * Loads a JSON stream from the content of @filename and parses it. See
- * json_parser_load_from_data().
- *
- * Return value: %TRUE if the file was successfully loaded and parsed.
- *   In case of error, @error is set accordingly and %FALSE is returned
- */
-gboolean
-json_parser_load_from_file (JsonParser   *parser,
-                            const gchar  *filename,
-                            GError      **error)
+static gboolean
+json_parser_load (JsonParser   *parser,
+                  const gchar  *data,
+                  gsize         length,
+                  GError      **error)
 {
-  GError *internal_error;
-  gchar *data;
-  gsize length;
-  gboolean retval = TRUE;
-
-  g_return_val_if_fail (JSON_IS_PARSER (parser), FALSE);
-  g_return_val_if_fail (filename != NULL, FALSE);
-
-  internal_error = NULL;
-  if (!g_file_get_contents (filename, &data, &length, &internal_error))
-    {
-      g_propagate_error (error, internal_error);
-      return FALSE;
-    }
-
-  if (!json_parser_load_from_data (parser, data, length, &internal_error))
-    {
-      g_propagate_error (error, internal_error);
-      retval = FALSE;
-    }
-  
-  g_free (data);
-
-  return retval;
-}
-
-/**
- * json_parser_load_from_data:
- * @parser: a #JsonParser
- * @data: the buffer to parse
- * @length: the length of the buffer, or -1
- * @error: return location for a #GError, or %NULL
- *
- * Loads a JSON stream from a buffer and parses it. You can call this function
- * multiple times with the same #JsonParser object, but the contents of the
- * parser will be destroyed each time.
- *
- * Return value: %TRUE if the buffer was succesfully parser. In case
- *   of error, @error is set accordingly and %FALSE is returned
- */
-gboolean
-json_parser_load_from_data (JsonParser   *parser,
-                            const gchar  *data,
-                            gssize        length,
-                            GError      **error)
-{
-  JsonParserPrivate *priv;
+  JsonParserPrivate *priv = parser->priv;
   JsonScanner *scanner;
   gboolean done;
   gboolean retval = TRUE;
   gint i;
 
-  g_return_val_if_fail (JSON_IS_PARSER (parser), FALSE);
-  g_return_val_if_fail (data != NULL, FALSE);
-
   json_parser_clear (parser);
 
-  if (length < 0)
-    length = strlen (data);
-
-  priv = parser->priv;
-
   scanner = json_scanner_create (parser);
   json_scanner_input_text (scanner, data, length);
 
-  parser->priv->scanner = scanner;
+  priv->scanner = scanner;
 
   g_signal_emit (parser, parser_signals[PARSE_START], 0);
 
@@ -990,8 +995,8 @@ json_parser_load_from_data (JsonParser   *parser,
                         if (symbols[i].token == expected_token)
                           symbol_name = symbol_names + symbols[i].name_offset;
 
-                      if (msg)
-                        msg = g_strconcat ("e.g. `", symbol_name, "'", NULL);
+                      if (!msg)
+                        msg = g_strconcat ("e.g. '", symbol_name, "'", NULL);
                     }
 
                   if (scanner->token > JSON_TOKEN_INVALID &&
@@ -1040,6 +1045,102 @@ json_parser_load_from_data (JsonParser   *parser,
   return retval;
 }
 
+/**
+ * json_parser_load_from_file:
+ * @parser: a #JsonParser
+ * @filename: the path for the file to parse
+ * @error: return location for a #GError, or %NULL
+ *
+ * Loads a JSON stream from the content of @filename and parses it. See
+ * json_parser_load_from_data().
+ *
+ * Return value: %TRUE if the file was successfully loaded and parsed.
+ *   In case of error, @error is set accordingly and %FALSE is returned
+ */
+gboolean
+json_parser_load_from_file (JsonParser   *parser,
+                            const gchar  *filename,
+                            GError      **error)
+{
+  JsonParserPrivate *priv;
+  GError *internal_error;
+  gchar *data;
+  gsize length;
+  gboolean retval = TRUE;
+
+  g_return_val_if_fail (JSON_IS_PARSER (parser), FALSE);
+  g_return_val_if_fail (filename != NULL, FALSE);
+
+  priv = parser->priv;
+
+  internal_error = NULL;
+  if (!g_file_get_contents (filename, &data, &length, &internal_error))
+    {
+      g_propagate_error (error, internal_error);
+      return FALSE;
+    }
+
+  g_free (priv->filename);
+
+  priv->is_filename = TRUE;
+  priv->filename = g_strdup (filename);
+
+  if (!json_parser_load (parser, data, length, &internal_error))
+    {
+      g_propagate_error (error, internal_error);
+      retval = FALSE;
+    }
+
+  g_free (data);
+
+  return retval;
+}
+
+/**
+ * json_parser_load_from_data:
+ * @parser: a #JsonParser
+ * @data: the buffer to parse
+ * @length: the length of the buffer, or -1
+ * @error: return location for a #GError, or %NULL
+ *
+ * Loads a JSON stream from a buffer and parses it. You can call this function
+ * multiple times with the same #JsonParser object, but the contents of the
+ * parser will be destroyed each time.
+ *
+ * Return value: %TRUE if the buffer was succesfully parser. In case
+ *   of error, @error is set accordingly and %FALSE is returned
+ */
+gboolean
+json_parser_load_from_data (JsonParser   *parser,
+                            const gchar  *data,
+                            gssize        length,
+                            GError      **error)
+{
+  JsonParserPrivate *priv;
+  GError *internal_error;
+  gboolean retval = TRUE;
+
+  g_return_val_if_fail (JSON_IS_PARSER (parser), FALSE);
+  g_return_val_if_fail (data != NULL, FALSE);
+
+  priv = parser->priv;
+
+  if (length < 0)
+    length = strlen (data);
+
+  priv->is_filename = FALSE;
+  g_free (priv->filename);
+
+  internal_error = NULL;
+  if (!json_parser_load (parser, data, length, &internal_error))
+    {
+      g_propagate_error (error, internal_error);
+      retval = FALSE;
+    }
+
+  return retval;
+}
+
 /**
  * json_parser_get_root:
  * @parser: a #JsonParser
@@ -1109,11 +1210,18 @@ json_parser_get_current_pos (JsonParser *parser)
  * @parser: a #JsonParser
  * @variable_name: return location for the variable name, or %NULL
  *
- * A JSON data stream might sometimes contain an assignment, even though
- * it would technically constitute a violation of the RFC. #JsonParser
- * will ignore the left hand identifier and parse the right hand value
- * of the assignment. #JsonParser will record, though, the existence of the
- * assignment in the data stream and the variable name used.
+ * A JSON data stream might sometimes contain an assignment, like:
+ *
+ * |[
+ *   var _json_data = { "member_name" : [ ...
+ * ]|
+ *
+ * even though it would technically constitute a violation of the RFC.
+ *
+ * #JsonParser will ignore the left hand identifier and parse the right
+ * hand value of the assignment. #JsonParser will record, though, the
+ * existence of the assignment in the data stream and the variable name
+ * used.
  *
  * Return value: %TRUE if there was an assignment, %FALSE otherwise. If
  *   @variable_name is not %NULL it will be set to the name of the variable
